//
// Image-Klasse zur Vorlesung 
// Distributed Algorithms for Image and Video Processing
// (c) Stephan Kopf, Universitt Mannheim, 2009
//
#ifndef IMAGE_H
#define IMAGE_H

#include<string>
#include<iostream>
#include <fstream>
using namespace std;


class Image 
{
public:

    // Construction/Destruction
    Image ();
    Image (int width, int height, int band);
    Image (string &filename);
    ~Image ();
	Image (const Image& other)
	{
		// initialize variables
		width  = 0;
		height = 0;
		band   = 3;
		data   = NULL;

		// copy values
		resize(other.width,other.height,other.band);
		for (int b=0; b<other.band; b++) {
			for (int y=0; y<other.height; y++) {
				for (int x=0; x<other.width; x++) {
					data[b][y][x] = other.data[b][y][x];
				} // for
			} // for
		} // for
	}

	// assignment
    Image& operator=(const Image &other) {
		if (this != &other) {
			resize(other.width,other.height,other.band);
			for (int b=0; b<other.band; b++) {
				for (int y=0; y<other.height; y++) {
					for (int x=0; x<other.width; x++) {
						data[b][y][x] = other.data[b][y][x];
					} // for
				} // for
			} // for
		} // if
        return *this;
    } // operator=
	



    // Interface
    void resize  (int width, int height, int band);
    void clear (unsigned char value);
	void read (string &filename);
	void write (string &filename) const;

	// analyse input streams
	void rmWhiteSpace(istream *is);
	char getChar(istream *is);
	int getInt(istream *is);


private:
	void allocData();
	void freeData();


public:
	// variables
	int width, height, band;
	unsigned char ***data;
	bool decodingError;
	

}; // class Image

#endif
